#!/bin/bash

# -------------------------------------------------------------------------------
# Filename:    check_proclist.sh
# Revision:    1.0
# Date:        2006/12/15
# Author:      Luigi Balzano
# Email:       info@sistemistica.it
# Website:     www.sistemistica.it
# Description: Plugin to check a list of processes.
# Notes:       This plugin uses the "ps -eo" command
# -------------------------------------------------------------------------------
# Copyright:   2006 (c) Luigi Balzano
# License:     GPL
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# you should have received a copy of the GNU General Public License
# along with this program (or with Nagios); if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA
# -------------------------------------------------------------------------------
# Credits go to Ethan Galstad for coding Nagios
# If any changes are made to this script, please mail me a copy of the changes
#

PROGNAME=`basename $0`
PROGPATH=`echo $0 | /bin/sed -e 's,[\\/][^\\/][^\\/]*$,,'`
REVISION=`echo '$Revision: 1.0 $' | sed -e 's/[^0-9.]//g'`
TIMESTAMP=`date +%Y%m%d%H%M%S`

. $PROGPATH/utils.sh

print_usage() {
        echo "Usage: $PROGNAME -h"
        echo "Usage: $PROGNAME -v"
        echo "Usage: $PROGNAME -f CONFIGFILE"
        echo ""
        echo "Path of CONFIGFILE must be relative to the program path"
        echo "(e.g. no path needed if config file is in the program dir)"
        echo ""
        echo "CONFIGFILE must contain rows in this form:"
        echo "EUSER:COMMAND:ARGS:MIN:MAX"
        echo ""
        echo "EUSER = Name of effective user (who execute command)"
        echo "COMMAND = Command executed"
        echo "ARGS = Arguments string to match"
        echo "MIN:MAX = Permitted range for number of occurrences found is MIN<=X<=MAX => valid range [MIN,MAX]"
        echo "If MIN is not specified permitted range is X<=MAX => valid range (-inf,MAX]"
        echo "If MAX is not specified permitted range is X>=MIN => valid range [MIN,inf)"
        echo "With MIN>MAX permitted ranges are X<=MIN and X>=MAX => valid range (-inf,MAX]U[MIN,inf)"
        echo ""
        echo "If at least one \"row check\" fails, an alarm is triggered"
}

print_help() {
        print_revision $PROGNAME $REVISION
        echo ""
        print_usage
        echo ""
        echo "Plugin to check a list of processes."
        echo ""
        support
}

# No command line arguments are supplied for this script. Print help

if [ "$#" -eq "0" ]; then
        print_usage
        exit $STATE_UNKNOWN
fi

# Print help with --help

if [[ ("$#" -eq "1") && ("$1" = "--help") ]]; then
        print_help
        exit $STATE_UNKNOWN
fi

unset CONFIGFILE

while getopts "vhf:" Option
do
  case $Option in
    h     ) print_help
            exit $STATE_UNKNOWN;;
    v     ) print_revision $PROGNAME $REVISION
            exit $STATE_UNKNOWN;;
    f     ) CONFIGFILE=$OPTARG;;
    *     ) print_usage
            exit $STATE_UNKNOWN;;   # DEFAULT
  esac
done

if [ ! -n "$CONFIGFILE" ]
  then
        echo "Unspecified argument"
        print_usage
        exit $STATE_UNKNOWN
fi

if [ ! -f $PROGPATH/$CONFIGFILE ]
  then
        echo "Config file not found"
        print_usage
        exit $STATE_UNKNOWN
fi

PSSTATUS=`ps -eo "%U %a" --width 4096 | sed 's/ /\t/' | awk 'NR>1 {printf ("%s\n", $0)}'`
VALID=0
INVALID=0
CRITICAL=0
CRITICALLIST=""
while [ 1 ]
do
  read CURRPROCESS || break
  FIRSTNBCHAR=$(echo $CURRPROCESS | sed 's/^ *//')
  FIRSTNBCHAR=${FIRSTNBCHAR:0:1}
  if [ "$FIRSTNBCHAR" != "#" ]
    then
      # Not a comment
      FIELDS=$(echo $CURRPROCESS | awk -F ":" '{print NF}')
      EUSER=$(echo $CURRPROCESS | cut -d":" -f1)
      COMMAND=$(echo $CURRPROCESS | cut -d":" -f2)
      ARGS=$(echo $CURRPROCESS | cut -d":" -f3)
      MIN=$(echo $CURRPROCESS | cut -d":" -f4)
      MAX=$(echo $CURRPROCESS | cut -d":" -f5)
      CHECKNUM=`expr $MIN + 0 1>/dev/null 2>&1` # Return code is 0 if the expression is neither null nor 0, 1 if the expression is null or 0, 2 for invalid expressions.
      CHECKMIN=$?
      CHECKNUM=`expr $MAX + 0 1>/dev/null 2>&1` # Return code is 0 if the expression is neither null nor 0, 1 if the expression is null or 0, 2 for invalid expressions.
      CHECKMAX=$?
      if [[ ("$FIELDS" -eq "5") && ("$CHECKMIN" -ne "2") && ("$CHECKMAX" -ne "2") && (("${#MIN}" -gt "0") || ("${#MAX}" -gt "0")) ]]
        then
          # Valid config row
          VALID=`expr $VALID + 1`
          FOUND=`echo "$PSSTATUS" | awk -F "\t" '
          BEGIN { found = 0; }
          {
            curruser=$1;
            sub(/ +$/, "", curruser);
            sub(/^ +/, "", curruser);
            currcmdline=$2;
            sub(/ +$/, "", currcmdline);
            sub(/^ +/, "", currcmdline);
            firstblank = index(currcmdline, " ");
            if (firstblank == 0) {
              currcmd = currcmdline;
              currargs = "";
            } else {
              currcmd = substr(currcmdline, 1, firstblank - 1);
              currargs = substr(currcmdline, firstblank + 1, length(currcmdline) - firstblank);
            }
            if ((index(curruser, euser) > 0) && (index(currcmd, command) > 0) && (index(currargs, args) > 0)) { found++; }
          }
          END { printf ("%s\n", found); }
          ' euser=$EUSER command=$COMMAND args=$ARGS`
          if [ "${#MIN}" -gt "0" ]
            then
              if [ "${#MAX}" -gt "0" ]
                then
                  # [MIN,MAX] or (-inf,MAX]U[MIN,inf)
                  if [ "$MIN" -gt "$MAX" ]
                    then
                      # (-inf,MAX]U[MIN,inf)
                      if [[ ("$FOUND" -gt "$MAX") && ("$FOUND" -lt "$MIN") ]]
                        then
                          CRITICAL=`expr $CRITICAL + 1`
                          CRITICALLIST="$CRITICALLIST, $FOUND found for $CURRPROCESS"
                      fi
                    else
                      # [MIN,MAX]
                      if [[ ("$FOUND" -lt "$MIN") || ("$FOUND" -gt "$MAX") ]]
                        then
                          CRITICAL=`expr $CRITICAL + 1`
                          CRITICALLIST="$CRITICALLIST, $FOUND found for $CURRPROCESS"
                      fi
                  fi
                else
                  # [MIN,inf)
                  if [ "$FOUND" -lt "$MIN" ]
                    then
                      CRITICAL=`expr $CRITICAL + 1`
                      CRITICALLIST="$CRITICALLIST, $FOUND found for $CURRPROCESS"
                  fi
              fi
            else
              if [ "${#MAX}" -gt "0" ]
                then
                  # (-inf,MAX]
                  if [ "$FOUND" -gt "$MAX" ]
                    then
                      CRITICAL=`expr $CRITICAL + 1`
                      CRITICALLIST="$CRITICALLIST, $FOUND found for $CURRPROCESS"
                  fi
                else
                  # MIN null and MAX null should be impossible
                  echo "UNKNOWN - Fatal error in plugin logic flow"
                  exit $STATE_UNKNOWN
              fi
          fi
        else
          INVALID=`expr $INVALID + 1`
      fi
  fi
done < $PROGPATH/$CONFIGFILE

#### Check criticals ####
if [ "$CRITICAL" -gt "0" ]
  then
    CRITICALLIST=`echo "$CRITICALLIST" | sed 's/^, //'`
    echo "CRITICAL - $CRITICAL checks failed ($CRITICALLIST)"
    exit $STATE_CRITICAL
fi

#### Check warnings ####
if [ "$INVALID" -gt "0" ]
  then
    echo "WARNING - $INVALID invalid rows found in config file"
    exit $STATE_WARNING
fi

echo "OK - $VALID successful checks performed"
exit $STATE_OK
